/* A simple iterative connection-oriented text server */
/* daemon9@netcom.com */

#include <sys/types.h>
#include <sys/socket.h>
#include <syslog.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#define FBUFSIZE	68
#define FILELOCALE	"/etc/textServ.txt"
#define QLEN		5		

#ifndef INADDR_NONE
#define INADDR_NONE	0xffffffff
#endif


extern int errno;
extern char *sys_errlist[];

u_short htons(), ntohs();
u_long htonl();


int main(){
	
	int passiveTCP(char *,int);	/* socket allocation occurs here */

	struct sockaddr_in fsin;	/* FROM address of client */
	char *service="textServ";	/* service name */
	char buf[1];			/* don't care what input is... */
	int msock,ssock;		/* server socket */
	int alen;			/* FROM address length */
	int fd;				/* file descriptor */
	char *fileBuf;			/* holds file stuffs */

	msock=passiveTCP(service,QLEN);	/* the master TCP socket */
	syslog(LOG_INFO,"Text Server daemon started");

	while(1){			/* the slave TCP socket */
		ssock=accept(msock,(struct sockaddr *)&fsin,&alen);
		if(ssock<0){
			fprintf(stderr,"accept failed",sys_errlist[errno]);
			syslog(LOG_ERR,"Text Server daemon failure: %m");
			exit(1);
		}
		fd=open(FILELOCALE,O_RDONLY);
		fileBuf=(char *)malloc(FBUFSIZE);
		while(read(fd,fileBuf,sizeof(fileBuf)))write(ssock,fileBuf,sizeof(fileBuf));
		close(ssock);
	}

}					/* End main */

/*	
	passiveTCP.c
*/

int passiveTCP(service,qlen)
char *service;
int qlen;
{
	int passiveSock(char *,char*,int);
	return passiveSock(service,"tcp",qlen);
}


int passiveSock(service,protocol,qlen)
char *service, *protocol;
int qlen;
{

	struct servent *pse;
	struct protoent *ppe;
	struct sockaddr_in sin;

	int s,type;

	bzero((char *)&sin,sizeof(sin));	/* Zero out IP address info */
	sin.sin_family=AF_INET;			/* INET address family */
	sin.sin_addr.s_addr=INADDR_ANY;		/* packets can arrive on any iface */

						/* service name -> port number */
	
	
	if(pse=getservbyname(service,protocol))sin.sin_port=htons(ntohs((u_short)pse->s_port));
	else if(!(sin.sin_port=htons((u_short)atoi(service)))){
		fprintf(stderr,"can't get %s service entry\n",service);
		syslog(LOG_ERR,"Text Server daemon failure: %m");
		exit(1);
	}		

	if(!(ppe=getprotobyname(protocol))){
		fprintf(stderr,"can't get %s protocol entry",protocol);
		syslog(LOG_ERR,"Text Server daemon failure: %m");
		exit(1);
	}
						/* choose socket type */

	if(!strcmp(protocol,"udp"))type=SOCK_DGRAM;
	else type=SOCK_STREAM;

						/* allocate a socket */

	if((s=socket(PF_INET,type,ppe->p_proto))<0){
		fprintf(stderr,"can't create socket: %s\n",sys_errlist[errno]);
		syslog(LOG_ERR,"Text Server daemon failure: %m");
		exit(1);
	}
						/* Bind socket to port */	

	if(bind(s,(struct sockaddr *)&sin, sizeof(sin))<0){
		fprintf(stderr,"can't bind to %s port: %s\n",service,sys_errlist[errno]);
		syslog(LOG_ERR,"Text Server daemon failure: %m");
		exit(1);
	}
	if(type==SOCK_STREAM && listen(s,qlen)<0){
		fprintf(stderr,"can't listen on %s port: %s\n",service,sys_errlist[errno]);
		syslog(LOG_ERR,"Text Server daemon failure: %m");
		exit(1);
	}
	return s;
}

